#Food web construction for Chen et al 2024 Oil and gas platforms degrade benthic invertebrate diversity and food web structure
#April 15,2024
#R version 4.2.3

rm(list=ls())

library(cheddar)
library(tidyverse)
library(stringr)

#Loading Trophicgroup_mastersheet
setwd("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb")
species_master <- read.csv("species_trophicgroup3.csv",fileEncoding="windows-1252")
species_master$RawName <- str_trim(species_master$RawName)
species_master$ValidName <- str_trim(species_master$ValidName)

#Loading UK Benthos database v5.17
setwd("D:/UoE/Programming/UKB-MMS")
df <- read.csv("Bio_Abundance_Final.csv",fileEncoding="windows-1252") #400292 rows
df$SPECIES <- str_trim(df$SPECIES,side="both")
df$SPECIES <- gsub(" spp.","",df$SPECIES)
length(unique(df$SPECIES)) #4586
df_species <- data.frame(species=df$SPECIES[!duplicated(df$SPECIES)]) #4586

#Match raw species name with WORMS-verified species name
common <- data.frame(SPECIES=df_species[df_species$species %in% species_master$RawName ==T,]) #4586
identical(df_species$species,common$SPECIES) #should be TRUE
species_master_trophic <- species_master[,c("ValidName","RawName","feeding_strat")]
trophic_common <- merge(common,species_master_trophic,by.x = "SPECIES",by.y="RawName",all.x=TRUE)

#Load bodymass data (Clare et al 2022)
setwd("D:/UoE/Programming/UKB-MMS/FoodWebBodyMass")
bodymass <- read.csv("BodyMassFinal.csv") #same as BodyMassFinal_add3.csv
bodymass$RawName <- str_trim(bodymass$RawName)
bodymass <- bodymass[,c("RawName","BodyMassFinal")]
common <- data.frame(SPECIES=df_species[df_species$species %in% bodymass$RawName==T,])
identical(df_species$species,common$SPECIES) #should be TRUE
bodymass_common <- merge(common,bodymass,by.x="SPECIES",by.y="RawName",all.x=TRUE)

#Concatenate all data
df <- merge(df,trophic_common,by="SPECIES")
unique(df$feeding_strat) #check if filter feeder/Feeder shows
#Add bodymass in match with species
df <- merge(df,bodymass_common,by ="SPECIES",all.x=TRUE) #df should still have 400292 obs
rm(bodymass_common,common,df_species,species_master_trophic,trophic_common,bodymass)
#Keep useful columns
df <- df[,c("SITE","ValidName","SPECIES","feeding_strat","MEAN","BodyMassFinal",
            "YEAR","MONTH","SEASON","LAT_PLAT","LONG_PLAT","DISTANCE","BEARING","DEPTH_PLAT",
            "MD0","SILT_CLAY","ORGANIC","TOT_HC_GC","AVG_OIL","N_ALKANE","CPI","PAH_4R_202",
            "PAH_4R_228","PAH_5R_252","PAH_6R_276","AVG.NAPH","AVG.PHEN","AVG.DIBENZ","BA",
            "BA_TOT","CD","CR","CU","NI","PB","VA","ZN","FE","HG")]

#abundance group by sum for df (sum up abundance of same species per SITE)
#replicated nodes could be multiple same species at the same SITE
df1 <- df %>%
  group_by(SITE,ValidName) %>% #group by sum (multiple same species at same SITE)
  summarise(MEAN=sum(MEAN),
            TROPHIC.GROUP=first(feeding_strat),
            BodyMassFinal=first(BodyMassFinal))
rm(df)

##Compilation of basal trophic interactions##
basal_feedingguild <- species_master %>%
  filter(feeding_strat %in% c("Filter Feeder","Detritivore","Grazer","Scavenger"))
unique(basal_feedingguild$feeding_strat) #det,sca,fil,gra
basal_feedingguild <- basal_feedingguild[,c("ValidName","feeding_strat")]
#Basal resources for detritivore: FPOM,bacteria,fungi,protists,microalgae,faeces
det <- data.frame(feeding_strat=rep("Detritivore",6),
                  resource=c("FPOM","bacteria","fungi",
                             "protists","microalgae","faeces"))
#Basal resources for filter feeder:FPOM,bacteria,fungi,microalgae
fil <- data.frame(feeding_strat=rep("Filter Feeder",4),
                  resource=c("FPOM","bacteria","fungi","microalgae"))
#Basal resources for grazer: microalgae,macroalgae 
gra <- data.frame(feeding_strat=rep("Grazer",2),
                  resource=c("microalgae","macroalgae"))
#Basal resources for scavenver:carrion,CPOM,faeces
sca <- data.frame(feeding_strat=rep("Scavenger",3),
                  resource=c("carrion","CPOM","faeces"))
feedingmatrix <- rbind(det,fil,gra,sca)
unique(feedingmatrix$resource)
rm(det,fil,gra,sca)

basalmatrix <- merge(basal_feedingguild,feedingmatrix,by="feeding_strat")
basalmatrix <- basalmatrix[,c("ValidName","feeding_strat","resource")]
colnames(basalmatrix) <- c("consumer","feeding_strat","resource")
basalmatrix <- basalmatrix[,c("consumer","resource")]
rm(basal_feedingguild,feedingmatrix)


##Compilation of Predator/Parasite trophic interaction ####
#Load preparfeeding_matrix
setwd("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb")
preparmatrix <- read.csv("MetaWeb_preparmatrix.csv",fileEncoding = "windows-1252") #77338 obs
preparmatrix <- preparmatrix[preparmatrix$UKBentho=="T",] #8948 obs
unique(preparmatrix$feeding_strat) #predator,parasite and ""
preparmatrix <- preparmatrix[preparmatrix$feeding_strat !="",] #8941 obs
unique(preparmatrix$feeding_strat)

#Interaction matrix for predators and parasites is compiled seperately
#predator matrix
prematrix <- preparmatrix[preparmatrix$feeding_strat=="Predator",]

#parasite matrix
parmatrix <- preparmatrix[preparmatrix$feeding_strat=="Parasite Fluid Feeders",]
parmatrix <- parmatrix[,c("consumer","validresource")]
parmatrix <- parmatrix[!duplicated(parmatrix),]
colnames(parmatrix) <- c("consumer","resource")

#add carrion to into predator's diet
predator_sca <- data.frame(feeding_strat=rep("Predator",1),
                           resource="carrion")
pre_feedingguild <- prematrix[,c("consumer","feeding_strat")]
pre_feedingguild <- pre_feedingguild[!duplicated(prematrix[,c("consumer","feeding_strat")]),]
prescamatrix <- merge(pre_feedingguild,predator_sca,by="feeding_strat")
prescamatrix$feeding_strat <- NULL

#combine predator matrix into predator scavenger basal matrix
prematrix <- prematrix[,c("consumer","validresource")]
prematrix <- prematrix[!duplicated(prematrix),]
colnames(prematrix) <- c("consumer","resource")
prematrix <- rbind(prematrix,prescamatrix)
prematrix <- prematrix[order(prematrix$consumer),]

#Combination of predator and parasite matrix
preparmatrix <- rbind(prematrix,parmatrix)
rm(prematrix,parmatrix,predator_sca,prescamatrix,pre_feedingguild)

#Compile entire trophic links (basal+prepar)
links <- rbind(basalmatrix,preparmatrix)
links$consumer <- str_trim(links$consumer,side="both")
links$resource <- str_trim(links$resource,side="both")

#Find replicate food web links
links$ID <- paste(links$consumer,links$resource)
length(unique(links$ID)) 

#remove repeated links (now links removed replicated rows:links=correct.links)
correct.links <- links[which(!duplicated(links$ID)),]
correct.links <- unique(correct.links)
length(unique(correct.links$ID))

#create basal species for subsequent food web construction
Addon <- c("FPOM","bacteria","fungi","protists","microalgae","carrion","CPOM","faeces","macroalgae") #9 basal resources
Addon.nodes <- cbind(data.frame(node=Addon),data.frame(M=rep(NA,9),N=rep(NA,9))) # M >0 & < Inf

#create unique SITE identified in the UK Benthos database to loop over all samples
SITECODE <- unique(df1$SITE)

#please use own working directory
#create empty folder named "communities"
dir.create("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb/carrionONLYfoodweb/communities/")

#Load
for(i in 1:length(unique(df1$SITE))){  
  web.df <- df1[df1$SITE==SITECODE[i],] #Here we use df1
  properties <- data.frame(title=paste(SITECODE[i]),M.units="gWW", N.units="m^-2") # Abundance is numbers per 0.1 square meter
  dir.create(paste("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb/carrionONLYfoodweb/communities/", SITECODE[i], sep=""), showWarnings=F)
  write.csv(properties, paste("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb/carrionONLYfoodweb/communities/", SITECODE[i], "/properties.csv", sep=""), row.names=F)
  
  nodes <- data.frame(node=web.df$ValidName,M=web.df$BodyMassFinal,N=web.df$MEAN)
  nodes <- nodes[nodes$N>0,] 
  nodes <- rbind(nodes,Addon.nodes)
  write.csv(nodes, paste("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb/carrionONLYfoodweb/communities/", SITECODE[i], "/nodes.csv", sep=""), row.names=F)
  
  trophic.links <- correct.links[correct.links$consumer %in% nodes$node ==T & correct.links$resource %in% nodes$node ==T,]
  write.csv(trophic.links, paste("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb/carrionONLYfoodweb/communities/", SITECODE[i], "/trophic.links.csv", sep=""), row.names=F)
}

setwd("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb/carrionONLYfoodweb")
MMS <- LoadCollection(dir="D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb") 

#Calculate food web properties
FoodWebProperties <- NULL

for (i in 1:length(unique(df1$SITE))){
  web.df <- df1[df1$SITE==SITECODE[i],]
  SITE <- SITECODE[i] #year and month in df_MMS.csv
  species.richness <- NumberOfNodes(MMS[[i]])-9 #subtract 9 basal resources out of nodes
  link.richness <- NumberOfTrophicLinks(MMS[[i]])
  linkage.density <- LinkageDensity(MMS[[i]])
  connectance <- DirectedConnectance(MMS[[i]])
  proportion.basal <- FractionBasalNodes(MMS[[i]])
  proportion.intermediate <- FractionIntermediateNodes(MMS[[i]])
  proportion.top <- FractionTopLevelNodes(MMS[[i]])
  mean.trophic.level <- mean(PreyAveragedTrophicLevel((MMS[[i]]))) 
  generality <- mean(TrophicGenerality(MMS[[i]])/Degree(MMS[[i]])) #InDegree
  generalitySD <- sd(NormalisedTrophicGenerality(MMS[[i]]))
  vulnerability <- mean(TrophicVulnerability(MMS[[i]])/Degree(MMS[[i]])) #OutDegree
  vulnerabilitySD <- sd(NormalisedTrophicVulnerability(MMS[[i]]))
  omnivory <- Omnivory(MMS[[i]]) 
  sub.df <- data.frame(SITE,species.richness,link.richness,linkage.density,connectance,
                       proportion.basal,proportion.intermediate,proportion.top,
                       mean.trophic.level,generality,generalitySD,vulnerability,vulnerabilitySD,omnivory)
  FoodWebProperties <- rbind(FoodWebProperties,sub.df)
}

#Write Food Web Properties
write.csv(FoodWebProperties,
          file.path("D:/UoE/Programming/BarrowBentho/UKBenthoslinks_recordproof/STOTEN foodweb",
                    "CarrionOnly_MetaWeb_foodwebproperties.csv"),row.names = F)
